﻿using AddinExamplesGeneral;
using System;
using System.Collections.Generic;
using System.Linq;
using Toolbox.Docking.Api.Control;
using Toolbox.Docking.Api.Objects;
using Toolbox.Docking.Api.Objects.Profiler;

namespace ToolboxAddinExamples.Profiler
{
    public class ProfilerAddinFactory : ITbProfilerFactory, ITbObjectFactoryView, ITbObjectFactoryExplain
    {
        private const string Caption = "(Test) ONS atoms";
        private static readonly Guid Guid = new Guid("B0B12C2A-2FC3-4DFA-BEE7-1EFA2B5F9B89");
        private static readonly Version Version = new Version(2, 1);
        private const string Description = "Dummy Profiler - Toolbox Docking Example";
        private const string Donator = "LMC";
        private const string Disclaimer = "For demonstrational purposes only.";
        private const string Authors = "Toolbox Development Team @LMC";
        private const string Url = "http://toolbox.oasis-lmc.org";

        public ProfilerAddinFactory()
        {
            ObjectId = new TbObjectId(Caption, Guid, Version);
            ObjectAbout = new TbObjectAbout(Description, Donator, Disclaimer, Authors, Url, Caption,
            //TODO: toolbox object documentation file?
            null, null);
            Flags = TbObjectFlags.None;
        }

        public bool InitFactory(IList<string> errorLog, out int? hash, ITbInitTask initTask)
        {
            /// Return an int value in order profiler to be cached.
            /// Just increase the velue if any modifications are done without chnging name or version of the profiler.
            hash = 1;
            return true;
        }

        public TbObjectFlags Flags { get; }

        public ProfilerFlags ProfilerFlags => ProfilerFlags.IndependentCategories;

        public TbObjectId ObjectId { get; }

        public TbObjectAbout ObjectAbout { get; }

        public string GetLiteratureForCategory(string category)
        {
            return string.Empty;
        }

        public ITbProfiler GetProfiler(ITbWorkTask task)
        {
            return new ProfilerAddin(AtomToCategoryDictionary);
        }

        public TbCategoryType ProfilerType => TbCategoryType.Empiric;
        public const string parentCaption = "Has Oxygen, Nitrogen or Sulfur";
        private static readonly IReadOnlyDictionary<string, string> AtomToCategoryDictionary =
            new Dictionary<string, string>
            {
                {"O", parentCaption+" >> "+"Has Oxygen"},
                {"N", parentCaption+" >> "+"Has Nitrogen"},
                {"S", parentCaption+" >> "+"Has Sulfur"}
            };

        private TbProfilingCategories _allCategories;

        public TbProfilingCategories AllCategories
        {
            get
            {
                if (_allCategories == null)
                {
                    List<TbProfilingCategoryNode> categories = new List<TbProfilingCategoryNode>();

                    TbProfilingCategoryNode parent = new TbProfilingCategoryNode(parentCaption, 3, System.Drawing.Color.Black, "",
                       null);
                    categories.Add(parent);
                    categories.AddRange(AtomToCategoryDictionary.Values.Select(x => new TbProfilingCategoryNode(x, 3, System.Drawing.Color.Black, parentCaption, null)));
                    categories.Add(new TbProfilingCategoryNode(ProfilerAddin.DefaultCategory, 3, System.Drawing.Color.Black, "",
                       null));

                    _allCategories = new TbProfilingCategories(categories, ProfilerAddin.DefaultCategory);
                }

                return _allCategories;
            }
        }

        public string ClientViewModule => AddinExamplesConsts.ProfilerViewModule;

        public string ClientExplainModule => AddinExamplesConsts.ProfilerExplainer;
    }
}
