﻿using System;

namespace Toolbox.Docking.Api.Objects.Profiler
{
    /// <summary>
    /// Profiling methods are separated into five groups based on the type of the 
    /// qualitative information used for chemical assessment. For more information see the F1 help 
    /// of your latest Toolbox installation.
    /// </summary>
    [Serializable]
    public enum TbCategoryType
    {
        /// <summary>
        /// Predefined categories are set of rules for grouping chemicals developed by 
        /// recognized institutions or organizations and they could be considered as “standard” categorization schemes. 
        /// </summary>
        Predefined = 0,

        /// <summary>
        /// Mechanistic categorization schemes consist of rules for identification of important chemical 
        /// characteristics based on published knowledge. Practically they relay on certain hypothesis 
        /// about the studied phenomenon. In this respect, their application is oriented for the phenomena 
        /// for which they were developed.
        /// </summary>
        Mechanistic = 1,

        /// <summary>
        /// Empiric categorization schemes include rules used to determine the chemical elements constituting 
        /// the target chemical, its chemical functionality and structural similarity.
        /// </summary>
        Empiric = 2,

        /// <summary>
        /// These are the profiling schemes created by the user. In any case, while creating a new profiler 
        /// the user may choose to place it in any of the above sections.
        /// </summary>
        Custom = 3,

        /// <summary>
        /// A repeated dose profiler used to identify the toxicological profiler of chemicals. 
        /// Used for repeated dose toxicity test data extracted from 
        /// database of Hazard Evaluation Support System (HESS). the profiler is developed by NITE, METI (Japan) 
        /// in cooperation with LMC
        /// </summary>
        Toxicological = 4,

        /// <summary>
        /// The Endpoint specific category profiler type determines substance properties that are related to specific
        /// adverse effects such as acute oral toxicity, bioaccumulation or mutagenicity
        /// </summary>
        CtEndpointSpecific = 5,

        /// <summary>
        /// Profiler assigning categories based on experimental data  
        /// </summary>
        ExperimentalData = 6,

        /// <summary>
        /// 
        /// </summary>
        Unknown = 100 //save place for more
    }
}