﻿using System;
using System.Collections.Generic;

namespace Toolbox.Docking.Api.Objects.Profiler
{
    /// <summary>
    /// This class contains the most basic description for a Toolbox profiler.
    /// </summary>
    [Serializable]
    public sealed class TbProfilerInfo : TbObjectInfo
    {
        /// <summary>
        /// Constructor for a profiler info object.
        /// </summary>
        /// <param name="objFlags">Object information flags.</param>
        /// <param name="profilerFlags">Profiler information flags.</param>
        /// <param name="profilerType">The type of profiler.</param>
        /// <param name="allCategories">List profiler's categories.</param>
        /// <param name="counterCategory">Counter category. Null or empty string if no counter category.</param>
        public TbProfilerInfo(TbObjectInfoFlags objFlags, ProfilerFlags profilerFlags, TbCategoryType profilerType,
            IEnumerable<string> allCategories, string counterCategory)
            : base(objFlags)
        {
            ProfilerFlags = profilerFlags;
            ProfilerType = profilerType;
            AllCategories = new List<string>(allCategories);
            CounterCategory = counterCategory;
        }

        /// <summary>
        /// Profiler object's type.
        /// </summary>
        public TbCategoryType ProfilerType { get; }

        /// <summary>
        /// List with all categories which the profiler can return
        /// </summary>
        public IReadOnlyList<string> AllCategories { get; }

        /// <summary>
        /// Category which is assigned if chemical not match explicitly defined categories.
        /// Null or empty string if this is not applicable for the profiler
        /// </summary>
        public string CounterCategory { get; }

        /// <summary>
        /// Description flags of the profiler.
        /// </summary>
        public ProfilerFlags ProfilerFlags { get; }
    }
}