﻿using System;

namespace Toolbox.Docking.Api.Data
{
    /// <summary>
    /// RelevancyType indicates how relevant is an object for a given designation (endpoint location with metadata)
    /// </summary>
    [Serializable]
    public enum TbRelevancy
    {
        /// <summary>
        /// None means that there is no information for relevancy type and the relevancy from a parent designation (if any) is assigned
        /// </summary>
        None,
        /// <summary>
        /// Plausible relevancy means that the object is moderately suited for the corresponding designation.
        /// It has a lesser priority than <see cref="Suitable"/> so if a parent designation is marked has a <see cref="Suitable"/> the latter will be accepted.
        /// </summary>
        Plausible,
        /// <summary>
        /// PlausibleAlways means plausible relevancy for the position no matter if the parent relecancy is <see cref="Suitable"/>
        /// </summary>
        PlausibleAlways,
        /// <summary>
        /// Suitable relevancy means that the object is suited for the corresponding designation.
        /// It has a greater priority than <see cref="Plausible"/> so if a parent designation is marked has a <see cref="Plausible"/> type 'Suitable' will be accepted.
        /// </summary>
        Suitable,
        /// <summary>
        /// Disabled means no relevancy for the position no matter the parent relevancies
        /// </summary>
        Disabled
    }
}
